// -*- c++ -*-

//
//      CondProb.java (Conditional Probability of Bayesian Net)
//                     Multi Layer Neural Network is included
//
//	Programmed by Y.Motomura(motomura@etl.go.jp)
//

import java.lang.*;
import java.io.*;
import java.util.*;
import java.net.*;

class CondProb {
	final static int Hidden = 10;
	int insize;
	int outsize;
	Node node;
	double inval[];
	double tmp[], h[];
//	double wih[maxIn][Hidden];
//	double who[Hidden][maxOut];
//	double tho[maxOut], th[Hidden];
	double wih[][];
	double who[][];
	double tho[], th[];

  CondProb(Node n, int ins, int outs) {
	  node = n; insize = ins; outsize = outs; 
	  wih = new double[insize][Hidden];
	  who = new double[Hidden][outsize];
	  tho = new double[outsize]; th = new double[Hidden];
	  inval = new double [insize]; tmp = new double[outsize];
	  h = new double[Hidden];
  }

  public void getOut(){
	int i,j,a;

      j = 0;
      for (Enumeration e = node.inNodes.elements(); e.hasMoreElements();) {
     	Node n = (Node)e.nextElement();
	a = Math.abs(n.Type);
	for(i=0; i<a; i++){
	  inval[j] = n.outval[i]; j++;
	}
      }
      if(node.Type>0){calcDiscrete();}
      else{calcContinuous();}
  }

  double Sigmoid(double x){
	return((double)1.0 / (1.0 + Math.exp( -1.0*(x))));
  }

  void calcContinuous(){
  int   i, j;
  double net;

    for( i = 0; i < Hidden; i++ ) {
      net=0.0;
      for(j = 0; j < insize; j++){
	net += wih[j][i] * inval[j];
      }
      h[i] = Sigmoid( net + th[i] );
    }
    for( i = 0; i < outsize; i++ ) {
      net = 0.0;
      for(j = 0; j < Hidden; j++){
        net += who[j][i] * h[j];
      }
      node.outval[i] = net + tho[i];
    }
  }

  void calcDiscrete(){
  int   i, j;
  double net, z1, z2;

    for( i = 0; i < Hidden; i++ ) {
      net = 0.0;
      for(j = 0; j < insize; j++){
        net += wih[j][i] * inval[j];
      }
      h[i] = Sigmoid( net + th[i] );
    }
  
    for( i = 0; i < outsize; i++ ) {
      net = 0.0;
      for(j = 0; j < Hidden; j++){
        net += who[j][i] * h[j];
      }
      tmp[i] = Sigmoid(net + tho[i]);
    }
    for(z1 = 0.0, i=0; i< outsize;i++){
      z1 += tmp[i];
    }
    for(z2 = 0.0, i=0; i< outsize;i++){
      tmp[i] = tmp[i] / z1;
      if(tmp[i]>0.95) tmp[i] = 1.0;
      if(tmp[i]<0.05) tmp[i] = 0.0;
      z2+=tmp[i];
    }
    for(i=0; i< outsize;i++){
	node.outval[i] = tmp[i]/z2;
    }
  }

  public void load() {
	   String fname;
	   InputStream is;
           StreamTokenizer st;

           int tType, i, j;

	   fname = new String(node.name + ".net");
           try {
	        is = new FileInputStream(fname);
            	st = new StreamTokenizer(is);
/* initialization of StreamTokenizer */
            	st.slashSlashComments(true);
            	st.eolIsSignificant(true);

// First line of net file includes Loop, Seed and other attribute data.
		while((tType = st.nextToken()) != StreamTokenizer.TT_EOL){
		}
		st.nextToken(); 
            	st.eolIsSignificant(false);
		for(j = 0; j < outsize ; j++){
                  	tho[j] = (double)st.nval;
			tType = st.nextToken(); 
		}
		i = 0;
            	while ((tType != StreamTokenizer.TT_EOF)&&(i < Hidden)) {
			for(j=0; j < outsize; j++){
				who[i][j]=(double)st.nval;
				tType = st.nextToken();
			}

			for(j=0; j < insize; j++){
				wih[j][i]=(double)st.nval;
				tType = st.nextToken();
			}
			th[i]=(double)st.nval;
			tType = st.nextToken();
			i++;
		}
		is.close();
	    } catch (IOException e) {
        	e.printStackTrace();
            }
//	    ContentsToString();
            return;
	}

	public void ContentsToString(){
		int i, j;
		StringBuffer s;
		s = new StringBuffer(4096);
		for(j = 0; j < outsize ; j++){
                  	s.append(Double.toString(tho[j])+"\n");
		}
            	for(i=0; i < Hidden; i++) {
			for(j=0; j < outsize; j++){
				s.append(Double.toString(who[i][j])+" ");
			}

			for(j=0; j < insize; j++){
				s.append(Double.toString(wih[j][i])+" ");
			}
			s.append(Double.toString(th[i])+"\n");
		}
		System.out.println(s.toString());
//	        System.out.println("-------------------------------");
  }
}



